@@ -38,6 +38,8 @@
 #define XY_COLOR_BLT_WRITE_RGB                  (1 << 20)
 #define XY_COLOR_BLT_DST_TILED                  (1 << 11)
 
+#define GEN8_XY_COLOR_BLT_CMD                   (CMD_2D | (0x50 << 22) | 0x05)
+
 /* BR13 */
 #define BR13_8                                  (0x0 << 24)
 #define BR13_565                                (0x1 << 24)
@@ -65,6 +67,7 @@
 struct intel_batchbuffer;
 
 #define ALIGN(i, n)    (((i) + (n) - 1) & ~((n) - 1))
+#define IS_ALIGNED(i, n) (((i) & ((n)-1)) == 0)
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 #define ARRAY_ELEMS(a) (sizeof(a) / sizeof((a)[0]))
@@ -73,6 +76,13 @@ struct intel_batchbuffer;
 #define True 1
 #define False 0
 
+#define ASSERT_RET(value, fail_ret) do {    \
+        if (!(value)) {                 \
+            assert(0);                      \
+            return fail_ret;                \
+        }                                   \
+    } while (0)
+
 #define SET_BLOCKED_SIGSET()   do {     \
         sigset_t bl_mask;               \
         sigfillset(&bl_mask);           \
@@ -246,6 +256,30 @@ struct intel_region
 #define PCI_CHIP_BAYTRAIL_M_4           0x0157
 #define PCI_CHIP_BAYTRAIL_D             0x0155
 
+#define PCI_CHIP_BROADWELL_MS_GT1       0x1602
+#define PCI_CHIP_BROADWELL_MS_GT2       0x1612
+#define PCI_CHIP_BROADWELL_MS_GT2PLUS   0x1622
+
+#define PCI_CHIP_BROADWELL_M_GT1_1      0x1606
+#define PCI_CHIP_BROADWELL_M_GT2_1      0x1616
+#define PCI_CHIP_BROADWELL_M_GT2PLUS_1  0x1626
+
+#define PCI_CHIP_BROADWELL_M_GT1_2      0x160B
+#define PCI_CHIP_BROADWELL_M_GT2_2      0x161B
+#define PCI_CHIP_BROADWELL_M_GT2PLUS_2  0x162B
+
+#define PCI_CHIP_BROADWELL_M_GT1_3      0x160E
+#define PCI_CHIP_BROADWELL_M_GT2_3      0x161E
+#define PCI_CHIP_BROADWELL_M_GT2PLUS_3  0x162E
+
+#define PCI_CHIP_BROADWELL_D_GT1_1      0x160A
+#define PCI_CHIP_BROADWELL_D_GT2_1      0x161A
+#define PCI_CHIP_BROADWELL_D_GT2PLUS_1  0x162A
+
+#define PCI_CHIP_BROADWELL_D_GT1_2      0x160D
+#define PCI_CHIP_BROADWELL_D_GT2_2      0x161D
+#define PCI_CHIP_BROADWELL_D_GT2PLUS_2  0x162D
+
 #define IS_G45(devid)           (devid == PCI_CHIP_IGD_E_G ||   \
                                  devid == PCI_CHIP_Q45_G ||     \
                                  devid == PCI_CHIP_G45_G ||     \
@@ -365,4 +399,30 @@ struct intel_region
 #define IS_GEN7(devid)          (IS_IVYBRIDGE(devid) || \
                                  IS_HASWELL(devid))
 
+
+#define IS_BDW_GT1(devid)       (devid == PCI_CHIP_BROADWELL_M_GT1_1 || \
+                                 devid == PCI_CHIP_BROADWELL_M_GT1_2 || \
+                                 devid == PCI_CHIP_BROADWELL_M_GT1_3 || \
+                                 devid == PCI_CHIP_BROADWELL_D_GT1_1 || \
+                                 devid == PCI_CHIP_BROADWELL_D_GT1_2 || \
+                                 devid == PCI_CHIP_BROADWELL_MS_GT1)
+
+#define IS_BDW_GT2(devid)       (devid == PCI_CHIP_BROADWELL_M_GT2_1 || \
+                                 devid == PCI_CHIP_BROADWELL_M_GT2_2 || \
+                                 devid == PCI_CHIP_BROADWELL_M_GT2_3 || \
+                                 devid == PCI_CHIP_BROADWELL_D_GT2_1 || \
+                                 devid == PCI_CHIP_BROADWELL_D_GT2_2 || \
+                                 devid == PCI_CHIP_BROADWELL_MS_GT2)
+
+#define IS_BDW_GT2PLUS(devid)   (devid == PCI_CHIP_BROADWELL_M_GT2PLUS_1 || \
+                                 devid == PCI_CHIP_BROADWELL_M_GT2PLUS_2 || \
+                                 devid == PCI_CHIP_BROADWELL_M_GT2PLUS_3 || \
+                                 devid == PCI_CHIP_BROADWELL_D_GT2PLUS_1 || \
+                                 devid == PCI_CHIP_BROADWELL_D_GT2PLUS_2 || \
+                                 devid == PCI_CHIP_BROADWELL_MS_GT2PLUS)
+
+#define IS_GEN8(devid)          (IS_BDW_GT1(devid) ||   \
+                                 IS_BDW_GT2(devid) ||   \
+                                 IS_BDW_GT2PLUS(devid))
+
 #endif /* _INTEL_DRIVER_H_ */
